--[[
This file is part of Exploding numbers - a puzzle game

Exploding numbers is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
]]

--[[

local function c(h, a)
	local r, g, b =
		tonumber(h:sub(1, 2), 16),
		tonumber(h:sub(3, 4), 16),
		tonumber(h:sub(5, 6), 16)
	local function round(n)
		return n == 0 and '0' or tostring(
			-- round to 3 decimal places
			math.floor(n * 1000 / 255 + .5) / 1000
		):gsub('^0*(.-)0*$', '%1') -- trim leading and trailing 0s
	end
	print(
		'Loaded color #' .. h .. ' as {' ..
		round(r) .. ', ' .. round(g) .. ', ' .. round(b) .. '}'
	)
	return {love.math.colorFromBytes(r, g, b, a)}
end


local function dump_c(color_scheme)
	print("title = '" .. color_scheme.title .. "',")
	for k, v in pairs(color_scheme) do if k ~= 'title' then
		local o = k .. " = {"
		for a = 1, #v do
			if a > 1 then o = o .. ', ' end
			o = o .. math.floor(v[a] * 1000 + .5) / 1000
		end
		print(o .. '},')
	end end
end


local function dump_desat_c(color_scheme)
	print("title = '" .. color_scheme.title .. "',")
	for k, v in pairs(color_scheme) do if k ~= 'title' then
		local avg = (v[1] + v[2] + v[3]) / 3
		local o = k .. " = {"
		for a = 1, #v do
			if a > 1 then o = o .. ', ' end
			o = o .. math.floor((v[a] + avg) * 1000 / 2 + .5) / 1000
		end
		print(o .. '},')
	end end
end
]]

local C1, C2, C3, C4 = {.667, .667, .667}, {.667, 0, .667}, {0, .667, .667}, {0, 0, 0}


colors = {
	bonus = 7, -- last 7 are bonus colors
	{
		title = 'day',
		menu_bg = {.7, .6, .4},
		menu_text = {0, 0, 0},
		menu_text_disabled = {.42, .361, .24},
		menu_highlight = {.7, .384, .352},
		bg = {.8, .8, .8},
		numbers = {0, 0, 0},
		neutral = {.6, .6, .6},
		odd = {.8, .5, .5},
		db4 = {.5, .8, .5},
		goal = {.6, .7, .9},
		selected = {1, 1, 1},
		naked_bg = {.7, .7, .7},
		naked_neutral = {.5, .5, .5},
		naked_odd = {.7, .2, .2},
		naked_db2 = {.2, .2, .2},
		naked_db4 = {.2, .6, .2},
		naked_goal = {.3, .35, .5},
		naked_indicator = {.5, .5, .5},
		heading = {0, 0, .5},
		progress = {.7, .6, .4},
	},
	{
		title = 'night',
		menu_bg = {.1, .1, .1},
		menu_text = {.75, .6, .8},
		menu_text_disabled = {.8, .5, .8, .4},
		menu_highlight = {.3, .1, .2},
		bg = {.15, .15, .2},
		neutral = {.2, .2, .3},
		numbers = {.8, .8, .8},
		odd = {.075, .075, .1},
		db4 = {.3, .1, .2},
		goal = {.4, .5, .1},
		indicator = {.8, .8, .8, .5},
		selected = {.25, .25, .25},
		naked_neutral = {.25, .25, .38},
		naked_odd = {.55, .55, .8},
		naked_db2 = {.8, .8, .8},
		naked_db4 = {.8, .4, .6},
		naked_selected = {.4, .4, .4},
		progress = {0, 0, 0},
		heading = {1, 1, 1},
		message_bg = {.3, .1, .2},
		message_text = {.75, .6, .8},
		warning_bg = {.8, .4, .6},
		warning_text = {.1, .1, .1},
	},
	{
		title = 'vivid',
		menu_bg = {.6, .5, .7},
		menu_text = {0, 0, 0},
		menu_text_disabled = {.4, .3, .5},
		menu_highlight = {.7, .4, .7},
		bg = {.23, .23, .33},
		neutral = {.35, .35, .45},
		numbers = {0, 0, 0},
		odd = {.8, .2, .2},
		db2 = {.5, .5, .6},
		db4 = {.5, .8, .4},
		goal = {.6, .4, .8},
		indicator = {.5, .5, .6},
		selected = {.77, .77, .87},
		naked_indicator = {.5, .5, .6},
		progress = {.5, .8, .4},
		help_bg = {.2, .2, .3},
		help_text = {.8, .8, .8},
		heading = {.8, .9, .6},
	},
	{
		title = 'oled',
		menu_bg = {0, 0, 0},
		menu_text = {.45, .4, .35},
		menu_text_disabled = {.175, .2, .225},
		menu_highlight = {.45, .4, .35},
		bg = {0, 0, 0},
		neutral = {.175, .2, .225},
		db2 = {.45, .4, .35},
		odd = {.175, .2, .225},
		db4 = {.9, .2, .0},
		goal = {0, 0, 0},
		goal_numbers = {.45, .4, .35},
		selected = {.9, .2, .0},
		selected_numbers = {.7, .7, .7},
		naked_goal = {.9, .66, .11},
		naked_selected = {.7, .7, .7},
		progress = {.9, .2, .0},
		help_bg = {0, 0, 0},
		help_text = {.45, .4, .35},
		heading = {.7, .7, .7},
		message_bg = {.7, .7, .7},
		warning_bg = {.9, .2, .0},
	},
	{
		title = 'vga',
		menu_text = {.173, .196, .243},
		menu_bg = {.573, .612, .651},
		menu_text_disabled = {.757, .765, .78},
		menu_highlight = {.6, .3, .45},
		bg = {.212, .192, .22},
		neutral = {.165, .149, .165},
		numbers = {1, 1, 1},
		odd = {.216, .212, .616},
		db2 = {.608, .267, .569},
		db4 = {.643, .275, .275},
		goal = {.3, .525, .35},
		indicator = {.384, .36, .35},
		selected = {.384, .36, .35},
		naked_neutral = {0, 0, 0},
		naked_odd = {.345, .341, .992},
		naked_db2 = {.992, .439, .925},
		naked_db4 = {.988, .431, .431},
		naked_indicator = {.384, .36, .35},
		naked_selected = {.906, .871, .749},
		progress = {.643, .275, .275},
		help_text = {.757, .765, .78},
		heading = {.906, .89, .871},
		message_bg = {.573, .612, .651},
		message_text = {0, 0, 0},
		warning_bg = {.643, .275, .275},
	},
	{
		title = 'ega',
		menu_bg = {0, .667, 1},
		menu_text = {1, 1, 1},
		menu_text_disabled = {0, .333, .667},
		menu_highlight = {1, .667, 0},
		bg = {.333, .333, .333},
		neutral = {0, 0, 0},
		numbers = {1, 1, 1},
		odd_numbers = {0, .667, 1},
		db4_numbers = {1, .667, 0},
		goal = {0, .667, 1},
		selected = {1, .667, 0},
		indicator = {.667, .667, .667},
		naked_bg = {0, 0, 0},
		naked_neutral = {.333, .333, .333},
		naked_odd = {0, .667, 1},
		naked_db2 = {.667, .667, .667},
		naked_db4 = {1, .667, 0},
		naked_goal = {.333, .333, .333},
		naked_selected = {1, 1, 1},
		naked_indicator = {.333, .333, .333},
		progress = {.667, .667, .667},
		help_bg = {.333, .333, .333},
		help_text = {1, 1, 1},
		heading = {1, .667, 0},
		message_bg = {0, .667, 1},
		message_text = {0, 0, 0},
		warning_bg = {1, .667, 0},
	},
	{
		title = 'fire',
		menu_bg = {.141, .043, .067},
		menu_text = {.812, .329, .196},
		menu_text_disabled = {.231, .216, .271},
		menu_highlight = {.42, .333, .475, .5},
		bg = {.435, .223, .223},
		neutral = {.3, .092, .15},
		numbers = {.223, .092, .105},
		odd = {.711, .393, .242},
		db2 = {.973, .337, .035},
		db2_numbers = {.223, .092, .105},
		db4 = {.996, .388, .008},
		db4_numbers = {.988, .812, .31},
		goal = {.42, .333, .475},
		goal_numbers = {.98, .647, .004},
		indicator = {.223, .092, .105},
		selected = {.992, .714, .012},
		selected_numbers = {.511, .215, .188},
		naked_bg = {.435, .223, .223},
		naked_neutral = {.629, .387, .321},
		naked_db4 = {.988, .812, .31},
		naked_goal = {.1, 0, 0},
		naked_indicator = {.223, .092, .105},
		progress = {.843, .216, .2},
		help_bg = {.176, .078, .129},
		help_text = {.812, .329, .196},
		heading = {.988, .631, 0},
		message_bg = {.2, .188, .302},
		message_text = {.992, .365, .059},
		warning_bg = {.847, .094, .051},
		warning_text = {.604, .004, .086},
	},
	{
		title = 'ice',
		menu_bg = {.172, .247, .369},
		menu_text = {.276, .372, .502},
		menu_text_disabled = {.184, .29, .443},
		menu_highlight = {.367, .483, .636},
		bg = {.367, .483, .636},
		neutral = {.42, .544, .675},
		numbers = {.142, .215, .319},
		db2 = {.294, .541, .804},
		db4 = {.271, .592, .882},
		db4_numbers = {.813, .852, .905},
		goal = {.813, .852, .905},
		goal_numbers = {.09, .388, .827},
		indicator = {.813, .852, .905},
		selected = {.892, .925, .956},
		selected_numbers = {.47, .531, .572},
		naked_bg = {.172, .247, .369},
		naked_odd = {.257, .4, .626},
		naked_db4 = {.778, .823, .899},
		naked_goal = {.09, .388, .827},
		naked_indicator = {.257, .4, .626},
		progress = {.2, .345, .518},
		help_bg = {.773, .824, .889},
		help_text = {.276, .372, .502},
		heading = {.145, .345, .604},
		message_bg = {.643, .727, .827},
		message_text = {.142, .215, .319},
		warning_bg = {.145, .345, .604},
		warning_text = {.918, .969, 1},
	},
	{
		title = 'candy',
		menu_bg = {1, .757, .749},
		menu_text = {.047, .243, .271},
		menu_text_disabled = {.722, .463, .576},
		bg = {.96, .82, .8},
		neutral = {.729, .706, .757},
		numbers = {.047, 0, .235},
		odd = {.314, .51, .635},
		db2 = {.737, .706, .282},
		db4 = {.341, .537, .341},
		goal = {.749, .453, .757},
		selected = {.627, .455, .427},
		progress = {.722, .463, .576},
		heading = {.49, .33, .3},
		message_bg = {.704, .768, .704},
		warning_bg = {.722, .463, .576},
	},
	{
		title = 'slime',
		menu_bg = {.643, .514, .369},
		menu_text = {.925, .741, .58},
		menu_text_disabled = {.714, .596, .467},
		menu_highlight = {.745, .6, .102},
		bg = {.447, .388, .196},
		neutral = {.62, .529, .255},
		numbers = {.224, .173, .106},
		odd = {.541, .463, .325},
		odd_numbers = {.192, .145, .082},
		db2 = {.714, .596, .467},
		db4 = {.745, .6, .102},
		db4_numbers = {.337, .271, .047},
		goal = {.514, .502, .078},
		selected = {.894, .443, .329},
		selected_numbers = {.592, .227, .173},
		naked_bg = {.286, .216, .071},
		naked_odd = {.541, .525, .388},
		naked_goal = {.514, .541, .078},
		naked_indicator = {.62, .529, .255},
		progress = {.894, .682, .082},
		help_bg = {.816, .722, .62},
		help_text = {.224, .173, .106},
		heading = {.412, .278, .11},
		message_bg = {.314, .212, .133},
		message_text = {.643, .514, .369},
		warning_bg = {.749, .239, .176},
		warning_text = {.949, .706, .353},
	},
	{
		title = 'cga',
		menu_bg = C1,
		menu_text = C4,
		menu_text_disabled = C2,
		menu_highlight = C3,
		bg = C4,
		neutral = C2,
		numbers = C4,
		odd_numbers = C1,
		db2 = C1,
		db2_numbers = C2,
		db4 = C3,
		db4_numbers = C4,
		goal = C2,
		goal_numbers = C3,
		selected = C3,
		selected_numbers = C1,
		naked_goal = C3,
		progress = C2,
		help_text = C1,
		heading = C3,
		message_bg = C1,
		message_text = C4,
		warning_bg = C2,
	},
	{
		title = 'acrylic',
		menu_bg = {.404, .533, .384},
		menu_text = {.85, .85, .85},
		menu_text_disabled = {.271, .369, .259},
		menu_highlight = {.121, .432, .312},
		bg = {.315, .345, .39},
		neutral = {.31, .32, .385},
		numbers = {.24, .293, .345},
		odd = {.03, 0, .02},
		db2 = {.608, .584, .267},
		db4 = {.584, .604, .624},
		goal = {.404, .533, .384},
		indicator = {.03, 0, .02},
		selected = {.6, .341, .518},
		naked_neutral = {.2, .21, .27},
		naked_indicator = {.404, .533, .384},
		progress = {.62, .341, .518},
		help_bg = {.584, .604, .624},
		help_text = {.24, .293, .345},
		heading = {.03, 0, .02},
		message_text = {0, 0, 0},
		warning_bg = {.6, .341, .518},
	},
	{
		title = 'virus',
		menu_bg = {.03, .108, .282},
		menu_text = {.1, .4, .953},
		menu_text_disabled = {.05, .308, .698},
		menu_highlight = {.05, .5, .5},
		bg = {.021, .043, .1},
		neutral = {.043, .085, .2, 0},
		numbers = {0, 0, 0},
		odd = {.4, .0, .0},
		db2 = {.56, .0, .0},
		db4 = {.72, .0, .0},
		goal = {0, 0, 0},
		goal_numbers = {.1, .4, .953},
		indicator = {.05, .308, .698},
		selected = {.03, .108, .282},
		selected_numbers = {.1, .4, .953},
		naked_neutral = {.155, .31, .72},
		naked_odd = {.56, .06, .06, .6},
		naked_db2 = {.72, .07, .07},
		naked_db4 = {.9, .1, .1},
		naked_goal = {0, 0, 1},
		naked_selected = {.155, .31, .72},
		progress = {.55, .06, .06},
		help_bg = {.043, .085, .2},
		help_text = {.155, .31, .72},
		heading = {.25, .5, 1},
		message_bg = {.282, .108, .03},
		message_text = {.953, .4, .1},
		warning_bg = {0, 0, 0},
		warning_text = {.07, .323, .953},
	},
	{
		title = 'sudoku',
		menu_bg = {.75, .77, .75},
		menu_text = {0, 0, 0},
		menu_text_disabled = {.5, .51, .5},
		menu_highlight = {0, .38, .6},
		bg = {.75, .77, .75},
		numbers = {.75, .77, .75},
		odd = {.75, .77, .75},
		odd_numbers = {0, 0, 0},
		db2 = {0, 0, 0},
		db4 = {0, .38, .6},
		goal = {.3, .1, .5},
		indicator = {0, 0, 0},
		selected = {.85, .88, .85},
		selected_numbers = {0, 0, 0},
		naked_odd = {0, 0, 0},
		naked_indicator = {0, 0, 0},
		naked_selected = {.85, 0, 0},
		progress = {.11, .66, .9},
		help_text = {0, 0, 0},
		heading = {0, .38, .6},
		message_bg = {0, .659, .42},
		message_text = {0, 0, 0},
		warning_bg = {.85, 0, 0},
	},
	{
		title = 'sakura',
		menu_bg = {1, .778, .713},
		menu_text = {.3, .23, .23},
		menu_text_disabled = {1, .98, .95},
		menu_highlight = {.3, .23, .23},
		selected = {.9, .1, .1},
		bg = {.95, .85, .85},
		neutral = {1, 1, 1},
		numbers = {.95, .85, .85},
		odd = {.23, .12, .1},
		db2 = {.23, .12, .1},
		db4 = {.23, .12, .1},
		goal = {.1, .6, .3},
		naked_bg = {1, .778, .713},
		naked_indicator = {1, 1, 1},
		progress = {1, 1, 1},
		help_text = {0, 0, 0},
		heading = {.5, .15, .15},
		message_bg = {.1, .6, .3},
		message_text = {1, 1, 1},
		warning_bg = {.85, .384, .352},
	},
}


function set_color(name)
	local c = colors[settings.color_scheme]
	local substitutions = {
		menu_highlight = 'menu_text_disabled',
		menu_highlight_text = 'menu_text',
		neutral = 'bg',
		odd = 'neutral',
		odd_numbers = 'numbers',
		db2 = 'neutral',
		db2_numbers = 'numbers',
		db4_numbers = 'numbers',
		indicator = 'neutral',
		goal_numbers = 'numbers',
		selected_numbers = 'numbers',
		naked_neutral = 'neutral',
		naked_odd = 'odd',
		naked_db2 = 'db2',
		naked_db4 = 'db4',
		naked_goal = 'goal',
		naked_indicator = 'naked_selected',
		naked_selected = 'selected',
		help_bg = 'bg',
		help_text = 'menu_text',
		message_bg = 'db4',
		message_text = 'numbers',
		warning_bg = 'odd',
		warning_text = 'message_text',
	}
	local default_colors = {
		warning_bg = {.8, .2, .2},
		message_bg = {.5, .8, .4},
	}
	love.graphics.setColor(
		c[name] or c[substitutions[name]] or
		c[substitutions[substitutions[name]]] or
		default_colors[name] or {0, 0, 0}
	)
end


function set_bg_color()
	colors.bg_color = settings.rectangles == 2 and
		colors[settings.color_scheme].naked_bg or
		colors[settings.color_scheme].bg
end


function appropriate_color(value, selected)
	if selected then
		return 'selected'
	elseif not value or value == '@' then
		return 'neutral'
	elseif value == current.goal then
		return 'goal'
	elseif value % 4 == 0 then
		return 'db4'
	elseif value % 2 == 0 then
		return 'db2'
	else
		return 'odd'
	end
end
