--[[
This file is part of Exploding numbers - a puzzle game

Exploding numbers is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
]]


-- replaces functions from the main game
-- old functions are saved as rr.old_xx
function hijack(mini)
	local h = mini.hijacked_love_functions
	for a = 1, #h do
		mini['old_' .. h[a]] = love[h[a]]
		love[h[a]] = mini[h[a]]
	end
	local h = mini.hijacked_globals
	if not h then return end
	for a = 1, #h do
		mini['old_' .. h[a]] = _G[h[a]]
		_G[h[a]] = mini[h[a]]
	end
end


-- restore hijacked functions and globals
function release(mini)
	local h = mini.hijacked_love_functions
	for a = 1, #h do
		love[h[a]] = mini['old_' .. h[a]]
	end
	local h = mini.hijacked_globals
	if not h then return end
	for a = 1, #h do
		_G[h[a]] = mini['old_' .. h[a]]
	end
end


help = {

text = function(self)
	local t = {
		general = {[[

*Exploding numbers* v0.992
Distributed under the terms of GNU GPL 3+ license (see below)


*GOAL*

Reach a certain number (written top left) by dividing and adding numbers in grid, according to the rules.


*RULES*

• Even numbers split when an empty tile *next to them* is clicked. One half is blown away from the clicked tile.*
]]
, -- figure 1
[[**
• Numbers divisible by four explode in four directions when clicked.*
]]
, -- figure 2
[[**
• The number that lands on another number is added to it.*
]]
, -- figure 3
[[**

*UNLOCKABLES*
]]
, -- table
[[**

*MAIN MENU*

*Goal* - The current level's goal is shown on this button, which also hides the menu.

*Level -* / *Restart* (F5) - restart level if the level is changed, or skip to the previous level
*Level +* / *Next* (F6) - skip to the next level

*Undo* / *Back* and *Redo* / *Fwd* - review the steps of either the built-in solution, or your own actions
*Solve* / *Play* (F7) toggles the built-in solution. While the solution is shown, any clicks in grid won't be remembered.

]] .. ( settings.editing and
[[*Edit* - start editing the current level (built-in levels will be copied first).
]] or '') .. ( WEB and
[[*Save* (F2) - save current progress.
]] or '') .. [[


*KEYBOARD*

Anywhere:
*Esc* - exit submenu]] .. ( WEB and '' or [[ or quit game
*q* - quit game immediately]] ) .. [[

(*Shift* +) *c* - change color scheme

In help:
*F1* / *r* - exit help
*arrows*, *keypad*, *hjkl*, *PageUp*, *PageDown*, *Home*, *End*, *Space* and *b* - scroll the text

In grid:
*arrows* / *hjkl* / *keypad* - select tile
*Enter* / *Space* / *keypad 5* - click on selected tile

In menu:
*Shift* + *direction* - scroll menu if larger than screen
Keys are hinted on buttons by big or capital letters in their titles.


*MOUSE / TOUCH*

In grid:
*touch* / *click* - select
*Ctrl* + *wheel* / *pinch* gesture - set minimum zoom level (see *Settings*)
*drag* / (*Shift* +) *wheel* - scroll grid (when zoomed)

In menu:
*touch* / *click* - select
*drag* / *wheel* - scroll menu (if not entirely shown)
*Ctrl* + *wheel* - resize menu buttons


*LICENSES*

*Exploding numbers* - a puzzle game
Copyright © 2022, 2023, 2024, 2025 Pajo (xpio]] .. string.char(64) .. [[tut.by)

This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program. If not, see <*http://www.gnu.org/licenses/*>.


*Publica Ignominia* font (*https://m-casanova.github.io/PublicaIgnominia/*) is licensed under the SIL Open Font License, Version 1.1.

*Love* framework (*https://love2d.org*) is licensed under the zlib/libpng license.
*]]},

		editing = {[[

*EDITING*

When editing a built-in level, it will first be copied as a new custom level. Custom levels are marked with *º*.

While in Edit mode, clicking on the grid places new numbers. Click a number with the same value to remove it.
*Value* - choose a value
*Goal* - set the goal for the level
*Step* - for *Value* and *Goal* buttons

*Copy* - save the current state as a new level
*Save* - save as the current level
These buttons exit the editing mode.

*Clear* - empty the level
*Rewind* - undo all changes
Both actions can be reversed by *Undo* and *Redo*.
Saving a cleared level will delete it. Saving a rewound level will make no changes and keep the edited level's solution, if any.

*Export* - dump the current state of the edited level to clipboard, standard output and a file in Love's directory, without saving it among levels
See *Settings* section for the directory's path.
*Import* - import a level from clipboard as a new level
The level in clipboard should be delimited by *{* and *}*.
]]},

		settings = {[[

*SETTINGS MENU*

*Editing* - toggle *Edit* button in the main menu
*Full screen* - toggle full screen / windowed mode
*Buttons* - change menu size between 60 (small) and 100 (big)
*Marks* - mark divisible numbers with dots, the selection with underline and the goal with "!"
*Color scheme*, *Shape* - change the visual appearance

Current state, settings, custom levels and list of solved levels are saved in a file called *save*. The last exported level is saved in a readable file called *exported_level*. These files are in *]] .. love.filesystem.getSaveDirectory() .. [[* directory.
*Reset* - deletes them and exits the game

*Minimal zoom*
The entire level is visible by default, which may result in tiny numbers. This option sets a limit to zooming-out. It can also be set with pinching gesture, or *Ctrl* + *wheel* at the grid. The minimum circle size, if smaller than default, will be shown in the top-left circle. In case some parts of the level end up off screen, use *arrows*, (*Shift* +) *wheel*, or dragging to pan the view.
]]}
	}
	local colored_text = {}
	for fragment = 1, #t[self.context] do
		colored_text[fragment] = {}
		for match in t[self.context][fragment]:gmatch('(.-)%*') do
			table.insert(colored_text[fragment], match)
			table.insert(colored_text[fragment], (#colored_text[fragment] % 4 == 1)
				and self.heading_c or self.text_c
			)
		end
	end
	return colored_text
end,


hijacked_love_functions = { 'draw', 'resize', 'update', 'mousepressed',
	'mousemoved', 'mousereleased', 'wheelmoved', 'keypressed'
},


dragging = {
	start_over = function(self, old_speed)
		self.started = love.timer.getTime()
		self.last_moved = self.started
		self.last_y = help.y
		if old_speed then
			self.old_speed = self.speed
		else
			self.old_speed = 0
		end
	end,
},


draw_gfx = function(self, h)
	local fh = self.font_height
	local vpad = math.floor(fh * 1.5 + .5)
	function circle(number, color, x, y)
		set_color(color)
		if settings.rectangles == 1 then
			love.graphics.rectangle('fill', x - fh * .2, y + fh * .05, fh * .9, fh * .9)
		else
			love.graphics.circle('fill', x + fh / 4, y + fh / 2, fh / 2)
		end
		set_color(color .. '_numbers')
		-- tweak position
		if number == '@' then
			x, y = x - fh * .1, y - fh * .07
		else
			x = x - fh * .05
		end
		love.graphics.print(number, x, y)
	end
	function arrow(x, y)
		set_color'help_text'
		love.graphics.setLineWidth(2)
		love.graphics.line(x, y + fh / 2, x + fh / 2, y + fh / 2)
		love.graphics.line(
			x + fh / 4, y + fh / 4,
			x + fh / 2 , y + fh / 2,
			x + fh / 4, y + fh * 3 / 4
		)
	end
	-- fig 1
	circle('', 'selected', self.x + fh * 2, h + self.height1 + vpad)
	circle('2', 'db2', self.x + fh * 3, h + self.height1 + vpad)
	arrow(self.x + fh * 4.5, h + self.height1 + vpad)
	circle('@', 'selected', self.x + fh * 6, h + self.height1 + vpad)
	circle('1', 'odd', self.x + fh * 7, h + self.height1 + vpad)
	circle('1', 'odd', self.x + fh * 8, h + self.height1 + vpad)
	-- fig 2
	circle('4', 'selected', self.x + fh * 2, h + self.height2 + vpad)
	arrow(self.x + fh * 3.5, h + self.height2 + vpad)
	circle('1', 'odd', self.x + fh * 5, h + self.height2 + vpad)
	circle('1', 'odd', self.x + fh * 6, h + self.height2 + math.floor(fh * .5 + .5))
	circle('@', 'selected', self.x + fh * 6, h + self.height2 + vpad)
	circle('1', 'odd', self.x + fh * 6, h + self.height2 + math.floor(fh * 2.5 + .5))
	circle('1', 'odd', self.x + fh * 7, h + self.height2 + vpad)
	-- fig 3
	circle('3', 'odd', self.x + fh * 2, h + self.height3 + vpad)
	circle('', 'selected', self.x + fh * 3, h + self.height3 + vpad)
	circle('2', 'db2', self.x + fh * 4, h + self.height3 + vpad)
	circle('3', 'odd', self.x + fh * 5, h + self.height3 + vpad)
	arrow(self.x + fh * 6.5, h + self.height3 + vpad)
	circle('3', 'odd', self.x + fh * 8, h + self.height3 + vpad)
	circle('@', 'selected', self.x + fh * 9, h + self.height3 + vpad)
	circle('1', 'odd', self.x + fh * 10, h + self.height3 + vpad)
	circle('4', 'db4', self.x + fh * 11, h + self.height3 + vpad)
	-- table
	set_color'help_text'
	love.graphics.setLineWidth(1)
	local table_width, table_middle, table_h1, table_padding
	if screen.w >= self.x + fh * 13.5 + 2 then
		table_width = fh * 13.5
		table_middle = fh * 6
		table_h1 = 'Solve ... levels'
		table_padding = fh * 2.3
	else
		table_width = fh * 10
		table_middle = fh * 2.8
		table_h1 = 'Solve'
		table_padding = fh * .5
	end
	table_middle = math.floor(table_middle + .5)
	table_padding = math.floor(table_padding + .5)
	for a = 2, 5 do
		love.graphics.line(
			self.x + 1, h + self.height4 + fh * a,
			self.x + table_width + 1, h + self.height4 + fh * a
		)
	end
	love.graphics.line(
		self.x + table_middle, h + self.height4 + fh,
		self.x + table_middle, h + self.height4 + fh * 6
	)
	love.graphics.setLineWidth(2)
	love.graphics.rectangle('line',
		self.x, h + self.height4 + fh - 1,
		table_width + 2, fh * 5 + 2,
		2, 2
	)
	love.graphics.print('20%\n40%\n60%\n80%',
		self.x + table_padding, h + self.height4 + fh * 2
	)
	love.graphics.print('all levels\neditor\nshapes\nall color schemes',
		self.x + table_middle + fh * .3, h + self.height4 + fh * 2
	)
	set_color'heading'
	love.graphics.print(table_h1,
		self.x + math.floor(fh * .2 + .5), h + self.height4 + fh
	)
	love.graphics.print('to unlock ...',
		self.x + table_middle + math.floor(fh * .3), h + self.height4 + fh
	)
end,


draw = function() -- hijacked function
	love.graphics.setFont(help.font)
	love.graphics.clear(help.bg_c)
	love.graphics.setColor(1,1,1, 1)
	local h = math.floor(help.y + help.top_margin + .5)
	love.graphics.draw(help.formatted, help.x, h)
	if help.context == 'general' then help:draw_gfx(h) end
	help:draw_status()
end,


move = function(self, dy)
	screen.changed = true
	local oldy = self.y
	self.y = self.y + dy
	self.y = math.min(0, self.y)
	self.y = math.max(self.miny, self.y)
end,


keypressed = function(k) -- hijacked function
	if k == 'escape' or k == 'f1' or k == 'r' then
		help:quit()
	elseif k == 'q' then
		love.event.push('quit')
	elseif k == 'pagedown' or k == 'l' or k == 'space' or k == 'kp3' then
		help:move(help.top_margin + help.font_height - screen.h)
	elseif k == 'pageup' or k == 'h' or k == 'b' or k == 'kp9' then
		help:move(screen.h - help.font_height - help.top_margin)
	elseif k == 'end' or k == 'kp1' then
		help:move(- help.height)
	elseif k == 'home' or k == 'kp7' then
		help:move(help.height)
	elseif k == 'c' then
		local lk_id = love.keyboard.isDown
		menu['colors_' .. (
			(lk_id('lshift') or lk_id('rshift')) and 'prev' or 'next'
		)]:onclick()
		help:init_colors()
		help.formatted = love.graphics.newText(help.font, '')
		local txt = help:text()
		local h = 0
		for fragment = 1, #txt do
			help.formatted:addf(txt[fragment], help.w, 'left', nil, h)
			local h0 = help.formatted:getHeight() + h
			h = h0 + (fragment ~= 4 and 3 or 6) * help.font_height
		end
	end
end,


wheelmoved = function(x, y) -- hijacked function
	help:move(help.font_height * y * 2)
	help.dragging.speed = 0
end,


mousepressed = function(mx, my, button) -- hijacked function
	mx, my = love.graphics.inverseTransformPoint(mx, my)
	if my < settings.menu_size then
		if mx >= help.button_x then
			help:quit()
		else
			help.scroll = true
			help.dragging.speed = 0
			help.y = help.miny * mx / help.button_x
			help:move(0)
		end
	elseif button == 1 then
		help.dragging:start_over(true)
	end
end,


mousereleased = function(mx, my, button) -- hijacked function
	if help.dragging.started then
		help.dragging.speed = help.dragging.old_speed +
			(help.y - help.dragging.last_y) /
			(love.timer.getTime() - help.dragging.started)
	end
	help.scroll, help.dragging.started, help.dragging.last_moved = false, false, false
end,


mousemoved = function(x, y, dx, dy) -- hijacked function
	x, y = love.graphics.inverseTransformPoint(x, y)
	if help.dragging.started then
		help:move(dy)
		help.dragging.last_moved = love.timer.getTime()
		if (dy ~= 0) then
			-- on dragging opposite of old_speed
			if (dy > 0) ~= (help.dragging.old_speed > 0) then
				help.dragging.old_speed = 0
			end
			-- on direction change
			if (dy > 0) ~= ((help.y - help.dragging.last_y) > 0) then
				help.dragging:start_over()
			end
		end
	end
	if help.scroll then
		help.y = help.miny * x / help.button_x
		help:move(0)
	end
end,


draw_status = function(self)
	local h = settings.menu_size
	local v = (self.miny == 0) and 'F1' or math.floor(self.y / self.miny * 100 + .5) .. '%'
	if v ~= 'F1' then
		-- progress bar
		if settings.menu_size > screen.h / 8 then h = h / 2 end
		love.graphics.setColor(self.progress_bg_c)
		love.graphics.rectangle('fill',
			0, 0,
			screen.w, h
		)
		set_color'progress'
		love.graphics.rectangle('fill',
			0, 0,
			self.button_x * self.y / self.miny, h
		)
	end
	-- button
	set_color'menu_bg'
	love.graphics.rectangle('fill',
		self.button_x, 0,
		h, h
	)
	set_color'menu_highlight'
	love.graphics.rectangle('fill',
		self.button_x, 0,
		h, h / 10
	)
	set_color'menu_text'
	love.graphics.setFont(menu_font_small)
	love.graphics.print('Return', self.button_text_x, self.button_text_y)
	love.graphics.setFont(self.percent_font)
	if h == settings.menu_size then
		love.graphics.printf(v, self.button_x, self.button_text2_y, settings.menu_size, 'center')
	elseif v ~= 'F1' then
		love.graphics.print(v, self.percent_font_y, self.percent_font_y)
	end
end,


init_colors = function(self)
	local c = colors[settings.color_scheme]
	self.bg_c = c.help_bg or c.bg
	self.text_c = c.help_text or c.menu_text
	self.heading_c = c.heading
	self.progress_bg_c = {unpack(c.menu_bg)}
	self.progress_bg_c[4] = .6 -- add transparency
end,


init = function(self)
	-- determine context
	local old_context = self.context
	if menu.order == menu.settings_order then self.context = 'settings'
	elseif menu.order == menu.edit_order then self.context = 'editing'
	else self.context = 'general' end
	-- reset text height on context change
	if old_context ~= self.context then self.y = nil end

	screen.changed = true
	local FONT_SIZE = 20
	self.w = screen.w
	if self.w > 450 then
		self.x = 25
		self.w = self.w - 2 * self.x
	else
		self.x = 0
	end
	self:init_colors()
	self.font = love.graphics.newFont('font/medium.ttf', FONT_SIZE)
	self.font_height = self.font:getHeight()
	self.formatted = love.graphics.newText(self.font, '')
	local txt = self:text()
	local h = 0
	for fragment = 1, #txt do
		self.formatted:addf(txt[fragment], help.w, 'left', nil, h)
		local h0 = self.formatted:getHeight() + h
		self['height' .. (fragment < #txt and fragment or '')] = h0
		h = h0 + (fragment ~= 4 and 3 or 6) * self.font_height
	end
	self.button_x = screen.w - settings.menu_size
	self.top_margin = settings.menu_size
	self.miny = math.min(0, screen.h - self.height - 2 * self.top_margin)
	self.y = math.max(self.y or 0, self.miny)
	self.scroll = false
	love.keyboard.setKeyRepeat(true)
	help.dragging.speed = 0
	self.button_text_x = math.floor(.5 +
		self.button_x +
		(settings.menu_size - menu_font_small:getWidth('Return')) / 2
	)
	self.button_text_y = math.floor(settings.menu_size * .1 + .5)
	self.button_text2_y = math.floor(settings.menu_size / 2 + .5)
	self.percent_font =
		(menu_font_small:getHeight() < self.font_height)
		and self.font
		or menu_font_small
	self.percent_font_y = math.floor(.5 +
		settings.menu_size / 4 - self.percent_font:getHeight() / 2
	)
end,


main = function(self)
	hijack(self)
	self:init()
end,


update = function(dt) -- hijacked function
	check_rotation(dt)
	if help.scroll then return end
	if
		love.keyboard.isDown('down') or love.keyboard.isDown('j') or
		love.keyboard.isDown('kp2') or love.mouse.isDown(2)
	then
		help:move(- help.font_height / 10)
	elseif
		love.keyboard.isDown('up') or love.keyboard.isDown('k') or
		love.keyboard.isDown('kp8') or love.mouse.isDown(3)
	then
		help:move(help.font_height / 10)
	end
	-- apply and modify speed
	if help.dragging.started then
		-- on inactivity
		if love.timer.getTime() - help.dragging.last_moved > .2 then
			help.dragging:start_over()
		end
		-- decay old_speed
		help.dragging.old_speed = help.dragging.old_speed * .9
	elseif help.dragging.speed ~= 0 then
		help:move(help.dragging.speed * dt)
		help.dragging.speed = help.dragging.speed * .95
		if math.abs(help.dragging.speed) < .2 then
			help.dragging.speed = 0
		end
	end
end,


resize = function() -- hijacked function
	love.graphics.origin()
	local x, y, w, h = love.window.getSafeArea()
	-- prevent crash on very small window
	if w < 2 * settings.menu_size or h < 2 * settings.menu_size then return end
	screen.w, screen.h = w, h
	love.graphics.translate(x, y)
	resize_grid()
	help:init()
end,


quit = function(self)
	release(self)
	love.resize()
	love.keyboard.setKeyRepeat(false)
end,


} -- end of help
