--[[
This file is part of Exploding numbers - a puzzle game

Exploding numbers is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
]]


menu = {
	goal = {
		text = 'Goal',
		key = 'g',
		onclick = function()
			settings.menu_disabled = not settings.menu_disabled
			love.resize()
		end,
		value = function() return current.goal end
	},
	undo = {
		text = 'Undo',
		value = 'u',
		arrow = 'dl',
		key = 'u',
		validate = function(self)
			self.disabled = current.history.position == 0
			-- set text
			if current.history == personal_history[current.level] then
				self.text = 'Undo'
			else
				self.text = 'Back'
			end
		end,
		onclick = function() undo(-1) end,
	},
	redo = {
		text = 'Redo',
		value = 'r',
		arrow = 'dr',
		key = 'r',
		validate = function(self)
			-- check to disable
			self.disabled =
				current.history.position == #current.history
				or #current.history == 0
			-- set text
			if current.history == personal_history[current.level] then
				self.text = 'Redo'
			else
				self.text = 'Fwd'
			end
		end,
		onclick = function(self) undo(1) end,
	},
	history = {
		text = 'Solve',
		value = 'F7',
		key = 'f7',
		validate = function(self)
			-- prevent mess when skipping or editing levels
			-- while watching the solution
			if not levels[current.level].solution then
				self.disabled = true
				if current.history ~= personal_history[current.level] then
					-- this auto-exits Solve mode when no solution present
					-- hopefully safe
					current.history = personal_history[current.level]
				end
			else
				self.disabled = false
			end
			if current.history == personal_history[current.level] then
				self.text = 'Solve'
			else
				self.text = 'Play'
			end
		end,
		onclick = function(self)
			if current.history == personal_history[current.level] then
				current.history = copy_history(levels[current.level].solution, 0)
				self.text = 'Play'
			else
				current.history = personal_history[current.level]
				self.text = 'Solve'
			end
			undo(0)
		end,
	},
	help = {
		text = 'Help',
		value = 'F1',
		key = 'f1',
		onclick = function() help:main() end
	},
	level = {
		text_common = 'Level',
		text_left = '\nF5 -',
		value = '',
		arrow = 'dl',
		key = 'f5',
		validate = function(self)
			if current.history.position > 0 then
				self.text_left = 'Restart\nF5'
				self.text_common = nil
			else
				self.text_common = 'Level'
				self.text_left = '\nF5 -'
			end
		end,
		onclick = function()
			init_level(current.level - (current.history.position > 0 and 0 or 1))
			resize_grid()
		end
	},
	skip = {
		text_right = '\n+ F6',
		arrow = 'dr',
		value = function() return level_name(current.level) end,
		common_value = true,
		key = 'f6',
		validate = function(self)
			self.text_right = current.history.position > 0 and 'Next\nF6' or '\n+ F6'
		end,
		onclick = function()
			init_level(current.level + 1)
			resize_grid()
		end
	},
	colors_prev = {
		text_common = 'Color scheme',
		text_left = '\nC -',
		value = '',
		arrow = 'dl',
		-- key is handled globally
		onclick = function()
			local max_colors = get_progress() < .8 and (#colors - colors.bonus) or #colors
			settings.color_scheme = (settings.color_scheme - 2) % max_colors + 1
			set_bg_color()
			screen.changed = true
		end
	},
	colors_next = {
		text_right = '\n+ c',
		value = function () return colors[settings.color_scheme].title end,
		common_value = true,
		arrow = 'dr',
		-- key is handled globally
		onclick = function()
			local max_colors = get_progress() < .8 and (#colors - colors.bonus) or #colors
			settings.color_scheme = settings.color_scheme % max_colors + 1
			set_bg_color()
			screen.changed = true
		end
	},
	reset_toggle = {
		text = 'Reset',
		value = 'F10',
		arrow = 'ur',
		key = 'f10',
		onclick = function(self)
			if self.value == 'n' then
				menu.order = menu.settings_order
				self.text = 'Reset'
				self.value = 'F10'
				self.key = 'f10'
				self.arrow = 'ur'
				screen.messages = nil
				resize_menu()
			else
				menu.order = menu.reset_order
				self.text = nil
				self.value = 'n'
				self.key = 'n'
				self.arrow = nil
				resize_menu()
				new_message('This will delete settings, custom and solved levels', true, 500)
			end
		end
	},
	confirm_reset = {
		text_common = 'Confirm reset?',
		value = 'y',
		key = 'y',
		onclick = function()
			del_game()
			current = nil
			love.event.push('quit')
		end
	},
	resize_buttons = {
		text = 'Buttons',
		value = function() return settings.menu_size end,
		key = 'b',
		onclick = function()
			settings.menu_size = (settings.menu_size == 60) and 100 or 60
			resize_menu_fonts()
			love.resize()
		end
	},
	marks = {
		text = 'Marks',
		value = function() return settings.marks and 'ön' or 'off' end,
		key = 'm',
		onclick = function()
			settings.marks = not settings.marks
		end
	},
	change_shape = {
		text = 'Shape',
		value = 'o',
		key = 'o',
		validate = function(self) self.disabled = get_progress() < .6 end,
		onclick = function()
			settings.rectangles = ((settings.rectangles or 0) + 1) % 3
			draw_playfield_shape = switch_playfield_shape(settings.rectangles)
			set_bg_color()
		end
	},
	fullscreen = {
		text = 'Full\nscreen',
		value = function() return screen.fullscreen and '+' or '.' end,
		key = 'f',
		arrow = 'ul',
		onclick = function() toggle_fullscreen() end
	},
	settings_toggle = {
		text = 'Settings',
		value = 's',
		key = 's',
		onclick = function(self)
			settings.menu_disabled = false
			menu.order = (menu.order == menu.settings_order) and
				menu.default_order or menu.settings_order
			self.text = (menu.order == menu.settings_order) and 'Return' or 'Settings'
			resize_menu()
		end
	},
	decrease_min_zoom = {
		text_common = 'Minimal zoom',
		text_left = '\nF2 -',
		value = '',
		arrow = 'dl',
		key = 'f2',
		onclick = function(self)
			-- prevent dragging while zooming out
			-- as it creates bad offsets
			screen.grid_drag[1] = nil
			resize_min_grid_by(- 10)
			resize_grid()
			screen.zoom_indicator_delay = 100
		end
	},
	increase_min_zoom = {
		text_right = '\n+ F3',
		value = function() return settings.min_grid or 0 end,
		arrow = 'dr',
		common_value = true,
		key = 'f3',
		onclick = function(self)
			resize_min_grid_by(10)
			resize_grid()
			screen.zoom_indicator_delay = 100
		end
	},
	save_game = {
		text = 'Save',
		value = 'F2',
		key = 'f2',
		onclick = function(self)
			save_game()
			new_message'Saved'
		end
	}
} -- end of menu table


menu.default_order = {
	menu.goal, menu.level, menu.skip,
	menu.undo, menu.redo, menu.history,
	menu.settings_toggle, menu.help
}

menu.settings_order = {
	menu.fullscreen,
	menu.colors_prev, menu.colors_next,
	menu.marks, menu.change_shape, menu.resize_buttons,
	menu.decrease_min_zoom, menu.increase_min_zoom,
	menu.reset_toggle, menu.help, menu.settings_toggle,
}
menu.reset_order = {
	menu.confirm_reset, menu.reset_toggle
}
menu.order = menu.default_order


function exit_menu_or_quit()
	if menu.order == menu.settings_order then
		menu.settings_toggle:onclick()
	elseif menu.order == menu.reset_order then
		menu.reset_toggle:onclick()
	elseif menu.order == menu.edit_order then
		menu.edit_stop:onclick()
	elseif
		not WEB and
		screen.messages and
		screen.messages[#screen.messages].message == 'Press again to quit'
	then
		love.event.push("quit")
	elseif not WEB then
		screen.messages = nil
		new_message('Press again to quit', true)
	end
	screen.changed = true
end


function resize_menu_disabled_button()
	-- resizes on goal change and menu font size change
	screen.menu_disabled_button = {
		x = 4,
		y = 4,
		w = menu_font_large:getWidth(current.goal),
		h = menu_font_large:getHeight(current.goal),
		ww = math.max(settings.menu_size, menu_font_large:getWidth(current.goal))
	}
	screen.changed = true
end


function try_disabling_menu_items()
	for a = 1, #menu.order do
		if menu.order[a].validate then menu.order[a]:validate() end
	end
	if screen then screen.changed = true end
end


function resize_menu_fonts()
	if menu_font_small then menu_font_small:release() end
	if menu_font_large then menu_font_large:release() end
	menu_font_small = love.graphics.newFont('font/medium.ttf', settings.menu_size / 4)
	menu_font_large = love.graphics.newFont('font/regular.ttf', settings.menu_size / 2)
	resize_menu_disabled_button()
end


function resize_menu()
	local function new_gradient(left)
		return love.graphics.newMesh({
			-- left
			{
				0, 0,
				0, 1,
				1, 1, 1, left and 1 or 0
			},
			{
				0, settings.menu_size,
				0, 0,
				1, 1, 1, left and 1 or 0
			},
			-- center
			{
				settings.menu_size * .25, 0,
				.5, 0,
				1, 1, 1, .6 -- a little less transparent
			},
			{
				settings.menu_size * .25, settings.menu_size,
				.5, 1,
				1, 1, 1, .6
			},
			-- right
			{
				settings.menu_size * .5, 0,
				1, 0,
				1, 1, 1, left and 0 or 1
			},
			{
				settings.menu_size * .5, settings.menu_size,
				1, 1,
				1, 1, 1, left and 0 or 1
			}
		}, "strip", "static")
	end -- of new_gradient

	screen.top_margin = (settings.menu_disabled and 0) or settings.menu_size
	if not (menu_font_small and menu_font_large) then resize_menu_fonts() end
	screen.buttons = {}
	for button = 1, #menu.order do
		local x = (button - 1) * settings.menu_size
		screen.buttons[button] = {
			x = x, y = 0, w = settings.menu_size, h = settings.menu_size,
			text_y = math.floor(settings.menu_size * .1 + .5),
			text2_y = math.floor(
				settings.menu_size -
				menu_font_large:getHeight(current.level) + .5
			)
		}
	end
	screen.buttons.dx = math.max(0,
		(screen.w - #screen.buttons * settings.menu_size) / 2
	)
	-- gradients
	menu.gradient_beginning = new_gradient(true)
	menu.gradient_end = new_gradient(false)
	-- text_layer
	menu.text_layer = love.graphics.newCanvas(screen.w, settings.menu_size)
end


function shift_menu(new_dx)
	local menu_w = #screen.buttons * settings.menu_size
	if screen.w - menu_w >= 0 then return end

--[[
	screen.buttons.dx = math.max(
		math.min(0, new_dx),
		math.min(dw / 2, dw)
	)
]]
	screen.buttons.dx = new_dx % menu_w
	screen.changed = true
end


function draw_menu()
	-- a small button which opens menu
	if settings.menu_disabled then
		love.graphics.setFont(menu_font_large)
		local s = screen.menu_disabled_button
		local function poly(style, c)
			set_color(c)
			love.graphics.polygon(style,
				s.x, s.y,
				s.x + s.ww, s.y,
				s.x + s.ww + s.h / 4, s.y + s.h / 2,
				s.x + s.ww, s.y + s.h,
				s.x, s.y + s.h
			)
		end
		poly('fill', 'menu_bg')
		love.graphics.setLineWidth(1)
		poly('line', 'menu_text')
		love.graphics.print(current.goal, 1.5 + s.x + (s.ww - s.w) / 2, s.y)
		-- hint
		love.graphics.setFont(menu_font_small)
		set_color('menu_text_disabled')
		love.graphics.print('g', 2 + s.x, s.y)
		return
	end
	-- the menu
	-- define buttons which should be highlighted
	local function should_highlight(i)
		return i == menu.goal or
			menu.order == menu.settings_order and
				i == menu.settings_toggle or
			i == menu.edit_stop
	end
	-- buttons function
	local function draw_button(a, dx)
		local item = menu.order[a]
		local s = screen.buttons[a]
		if should_highlight(item) then
			set_color('menu_highlight')
			love.graphics.rectangle('fill', dx + s.x, s.y, s.w, s.h / 10)
		end
		if item.arrow then
			c = s.h / 2.83 -- 2 * sqrt(2)
			local coordinates = {
				ul = {
					dx + s.x, s.y,
					dx + s.x + s.h / 2, s.y,
					dx + s.x, s.y + s.h / 2
				},
				ur = {
					dx + s.x + s.w, s.y + s.h / 2,
					dx + s.x + s.w, s.y,
					dx + s.x + s.w - s.h / 2, s.y
				},
				dr = {
					-1 + dx + s.x + s.w, s.y + s.h / 2,
					-1 + dx + s.x + s.w, s.y + s.h,
					-1 + dx + s.x + s.w - s.h / 2, s.y + s.h
				},
				dl = {
					dx + s.x, s.y + s.h / 2,
					dx + s.x, s.y + s.h,
					dx + s.x + s.h / 2, s.y + s.h
				},
				l = {
					dx + s.x, s.y + s.h / 2,
					dx + s.x + c, s.y + s.h / 2 - c,
					dx + s.x + c, s.y + s.h / 2 + c,
				},
				r = {
					dx + s.x + s.w, s.y + s.h / 2,
					dx + s.x + s.w - c, s.y + s.h / 2 - c,
					dx + s.x + s.w - c, s.y + s.h / 2 + c,
				}
			}
			set_color('menu_text_disabled')
			love.graphics.polygon('fill', coordinates[item.arrow])
		end
		local disabled = item.disabled
		if should_highlight(item) then
			set_color('menu_highlight_text')
		else
			set_color('menu_text' .. (disabled and '_disabled' or ''))
		end
		love.graphics.setCanvas(menu.text_layer)
		love.graphics.setFont(menu_font_small)
		local function print_text(pos) -- nil, '_left', '_right' or '_common'
			local t = item['text' .. pos]
			if not t then return end
			local tw = menu_font_small:getWidth(t)
			local offset
			if pos == '_left' then offset = s.w / 10
			elseif pos == '_right' then offset = s.w * .9 - tw
			elseif pos == '_common' then offset = s.w - tw / 2
			else offset = (s.w - tw) / 2 -- center
			end
			local tx = dx + s.x + offset
			-- use integer when animation stops
			tx = screen.menu_drag_speed and tx or math.floor(tx + .5)
			love.graphics.print(t, tx, s.text_y)
		end
		print_text'_common'; print_text'_left'; print_text''; print_text'_right'
		local v = item.value
		if v then
			disabled = disabled and
				(not item.common_value or menu.order[a - 1].disabled)
			if should_highlight(item) then
				set_color('menu_highlight_text')
			else
				set_color('menu_text' .. (disabled and '_disabled' or ''))
			end
			if type(v) == 'function' then v = v() end
			love.graphics.setFont(menu_font_large)
			local sx = item.common_value and s.x - settings.menu_size / 2 or s.x
			love.graphics.print(v,
				dx + sx + (s.w - menu_font_large:getWidth(v)) / 2,
				s.text2_y
			)
		end
		love.graphics.setCanvas()
	end -- of draw_button()
	-- actual drawing
	love.graphics.setScissor(screen.x, screen.y, screen.w, screen.h)
	-- background
	set_color('menu_bg')
	love.graphics.rectangle('fill',
		0, 0,
		screen.w, screen.top_margin
	)
	-- text gets drawn to separate layer
	local menu_w = #screen.buttons * settings.menu_size
	menu.text_layer:renderTo(love.graphics.clear)
	for a = 1, #menu.order do
		local dx = screen.buttons.dx
		local actual_x = screen.buttons[a].x + dx
		if actual_x > screen.w then
			draw_button(a, dx - menu_w) -- wrapped
		else
			draw_button(a, dx) -- not wrapped
			if actual_x > screen.w - settings.menu_size then
				-- duplicate the last, partially visible button
				draw_button(a, dx - menu_w)
			end
		end
	end
	-- menu overflow indicator
	set_color('menu_text_disabled')
	if screen.w < menu_w then
		love.graphics.draw(menu.gradient_beginning, 0, 0)
		love.graphics.draw(
			menu.gradient_end, screen.w - (settings.menu_size / 2), 0
		)
	end
	-- text layer
	love.graphics.setColor(1, 1, 1)
	love.graphics.setBlendMode("alpha", "premultiplied")
	love.graphics.draw(menu.text_layer, 0, 0)
	love.graphics.setBlendMode("alpha")
	--
	love.graphics.setScissor()
end
